import React, { lazy, useEffect, useState } from 'react'
import { TreeSelect, Form, Input, InputNumber, Radio, Modal, Switch, Popover, Button, Space, List } from 'antd';
import menuService from '@/request/service/menu';
import { useRequest } from '@/hooks/use-request';
import type { DefaultOptionType } from 'antd/es/select';
import type { Menu } from '@/models'
import { antdUtils } from '@/utils/antd';
import * as Icon from "@ant-design/icons";

const icon2Element = (name: string) => {
    return <React.Suspense fallback={<></>}>
        {React.createElement((Icon as any)[name], {
            style: { fontSize: '16px' }
        })}
    </React.Suspense>
}

interface MenuIconProps {
    value?: string;
    onChange?: (value: string) => void;
}

const MenuIcon: React.FC<MenuIconProps> = ({ value = '', onChange }) => {
    const [hovered, setHovered] = useState(false);
    const icons: string[] = [];
    for (const i in Icon) {
        if (i != 'default') icons.push(i)
    }
    const girdItems = icons.filter(i => i.endsWith('Outlined'))
    // if (value === '' || girdItems.indexOf(value) < 0) {
    //     value = 'MenuUnfoldOutlined'
    // }
    const triggerChange = (changedValue: string) => {
        onChange?.(changedValue);
        setHovered(false)
    };
    const renderIcons = () => {
        return (<List
            className='w-[400px]'
            grid={{ column: 10 }}
            dataSource={girdItems}
            pagination={{
                defaultPageSize: 80,
                showSizeChanger: false,
            }}
            size='small'
            renderItem={(item) => (
                <List.Item onClick={() => { triggerChange(item) }}>
                    <div className='w-[16px]'>{icon2Element(item)}</div>
                </List.Item>
            )}
        />)
    }

    const handleOpenChange = (isOpen: boolean) => {
        setHovered(isOpen)
    }

    return <Popover placement="bottom"
        content={renderIcons}
        open={hovered}
        onOpenChange={handleOpenChange}
        trigger="click">
        <Space>
            <Input value={value} />
            {(value && girdItems.indexOf(value) >= 0) ? icon2Element(value) : <></>}
        </Space>
    </Popover>
};



const layout = {
    labelCol: { span: 4, },
    wrapperCol: { span: 16 },
    bordered: false,
};

interface MenuEditorProps {
    visible: boolean;
    onCancel: (flag?: boolean) => void;
    onSave: (menu: Menu) => void;
    editData?: Menu | null;
    parentMenus: Menu[];
}


const MenuEditor: React.FC<MenuEditorProps> = (props) => {

    const { visible, onCancel, onSave, editData, parentMenus } = props;

    const { runAsync: updateMenu } = useRequest(menuService.updateMenu, { manual: true });
    const { runAsync: createMenu } = useRequest(menuService.createMenu, { manual: true });



    const isEdit = !!editData;

    const menu2Tree = (menu: Menu) => {
        const treeItem: DefaultOptionType = {
            label: menu.name,
            value: menu.id
        }
        if (menu.children && menu.children.length > 0) {
            treeItem.children = menu.children.filter(item => item.type !== 3).map(menu2Tree)
        }

        return treeItem;
    }
    const menuTree: DefaultOptionType[] = [{
        label: '主类目',
        value: 0,
        children: parentMenus.filter(item => item.type !== 3).map(item => {
            return menu2Tree(item)
        })
    }]

    const [saveLoading, setSaveLoading] = useState(false);
    const [form] = Form.useForm();

    useEffect(() => {
        if (visible) {
            if (editData) {
                form.setFieldsValue(editData);
            }
        } else {
            form.resetFields();
        }
    }, [visible]);

    const save = async () => {
        setSaveLoading(true);
        const menuValues = form.getFieldsValue();
        if (typeof (menuValues.status) === 'boolean') {
            menuValues.status = menuValues.status ? 0 : 1
        }
        const newValue = isEdit ? { ...editData, ...menuValues } : {
            permission: '',
            icon: '',
            keepAlive: true,
            createTime: new Date(),
            ...menuValues,
        }
        const [error, { msg, code }] = isEdit ? await updateMenu(newValue) : await createMenu(newValue);
        if (!error && code === 0) {
            onSave(newValue);
        } else {
            antdUtils.message?.open({
                type: 'error',
                content: msg ?? '操作失败',
            });
        }
        setSaveLoading(false);
    }


    return (
        <>
            <Modal
                open={visible}
                title={isEdit ? "编辑" : "新建"}
                width={640}
                onOk={save}
                onCancel={() => onCancel()}
                destroyOnClose
            >
                <Form
                    form={form}
                    {...layout}
                    onFinish={save}
                    labelCol={{ flex: '0 0 100px' }}
                    wrapperCol={{ span: 16 }}
                >
                    <Form.Item name="parentId" label="上级菜单" >
                        <TreeSelect treeData={menuTree} />
                    </Form.Item>

                    <Form.Item name="name" label="菜单名称"
                        rules={[
                            {
                                required: true,
                                message: '请输入菜单名称',
                            },
                        ]}
                    >
                        <Input />
                    </Form.Item>

                    <Form.Item name="type" label="菜单类型" >
                        <Radio.Group options={[
                            { value: 1, label: "目录" },
                            { value: 2, label: "菜单" },
                            { value: 3, label: "按钮" },
                            { value: 4, label: "菜单组" }
                        ]} optionType="button">
                        </Radio.Group>
                    </Form.Item>

                    <Form.Item name="icon" label="菜单图标" >
                        <MenuIcon />
                    </Form.Item>

                    <Form.Item name="path" label="路由地址"
                        rules={[
                            {
                                required: true,
                                message: '请输入路由地址',
                            }
                        ]}
                    >
                        <Input />
                    </Form.Item>

                    <Form.Item
                        noStyle
                        shouldUpdate={(prevValues, currentValues) => prevValues.type !== currentValues.type}
                    >
                        {({ getFieldValue }) =>
                            getFieldValue('type') === 2 ? (
                                <Form.Item name="component" label="组件地址" >
                                    <Input />
                                </Form.Item>
                            ) : null
                        }
                    </Form.Item>

                    <Form.Item name="sort" label="显示排序" >
                        <InputNumber min={1} max={100} />
                    </Form.Item>

                    <Form.Item name="status" label="菜单状态">
                        <Radio.Group options={[
                            { value: 0, label: "开启" },
                            { value: 1, label: "关闭" }
                        ]} optionType="default">
                        </Radio.Group>
                    </Form.Item>

                    <Form.Item name="visible" label="显示状态" valuePropName="checked">
                        <Switch checkedChildren="显示" unCheckedChildren="隐藏" />
                    </Form.Item>
                </Form>
            </Modal>
        </>
    )
}

export default MenuEditor;
